#include "my_main_loop.hpp"
#include "source/newtonian/test_1d/main_loop_1d.hpp"
#include "source/newtonian/one_dimensional/hdf5_diagnostics1d.hpp"
#include "source/misc/int2str.hpp"
#include "source/misc/vector_initialiser.hpp"
#include <fstream>

using simulation1d::main_loop;
using simulation1d::SafeTimeTermination;
using simulation1d::DiagnosticsFunction;
using simulation1d::WriteTime;
using diagnostics1d::write_snapshot_to_hdf5;
using std::pair;
using std::ofstream;
using std::endl;

namespace {
  class MultipleDiagnostics: public DiagnosticsFunction
  {
  public:

    MultipleDiagnostics
    (const vector<DiagnosticsFunction*>& diag_list):
      diag_list_(diag_list) {}

    ~MultipleDiagnostics(void)
    {
      for(size_t i=0;i<diag_list_.size();++i)
	delete diag_list_.at(i);
    }

    void diagnose(const hdsim1D& sim)
    {
      for(size_t i=0;i<diag_list_.size();++i)
	diag_list_.at(i)->diagnose(sim);
    }

  private:
    vector<DiagnosticsFunction*> diag_list_;
  };

  class IntervalSnapshot: public DiagnosticsFunction
  {
  public:

    IntervalSnapshot(double dt):
      counter_(0), t_next_(0), dt_(dt) {}

    void diagnose(const hdsim1D& sim)
    {
      if(sim.GetTime()>t_next_){
	write_snapshot_to_hdf5
	  (sim,
	   "snapshot_"+int2str(counter_)+".h5");
	t_next_ += dt_;
	counter_++;
      }
    }

  private:
    mutable int counter_;
    mutable double t_next_;
    const double dt_;
  };
}

void my_main_loop(hdsim1D& sim)
{
  const double tf = 2e-1;
  SafeTimeTermination term_cond(tf,1e7);
  MultipleDiagnostics diag
    (VectorInitialiser<DiagnosticsFunction*>
     (new WriteTime("time.txt"))
     (new IntervalSnapshot(tf/100))());
  main_loop(sim,
	    term_cond,
	    1,
	    &diag);
}
