#include "sim_data.hpp"
#include "create_grid.hpp"
#include <cmath>

SmoothInjection::SmoothInjection
(double D,
 double eta,
 double v_w,
 double GM,
 const PhysicalGeometry1D& pg):
  D_(D),
  eta_(eta),
  v_w_(v_w),
  GM_(GM),
  pg_(pg) {}

Conserved SmoothInjection::operator()
  (vector<double> const& vertices,
   vector<Primitive> const& cells,
   size_t point,
   double /*t*/,
   double /*dt*/) const
{
  const double volume = pg_.calcVolume(vertices.at(point+1))-
    pg_.calcVolume(vertices.at(point));
  const double mass = cells.at(point).Density*volume;
  const double r = 0.5*(vertices.at(point)+vertices.at(point+1));
  const double u = cells.at(point).Velocity.x;
  return Conserved(D_*pow(r,-eta_)*volume,
		   Vector2D(-1,0)*GM_*mass/pow(r,2),
		   D_*pow(r,-eta_)*volume*pow(v_w_,2)/2-GM_*mass*u/pow(r,2));
}

namespace {
  vector<double> logspace
  (double start,
   double end,
   size_t n)
  {
    vector<double> res(n);
    const double len = static_cast<double>(n);
    for(size_t i=0;i<n;i++){
      const double id = static_cast<double>(i);
      const double expnt = start + (end-start)*id/len;
      res.at(i) = pow(10,expnt);
    }
    return res;
  }
}

SimData::SimData(void):
  pg_(),
  sr_(),
  eos_(5./3.),
  rs_(),
  vm_(true),
  bc_(),
  source_(1,2.5,1,1,pg_),
  grid_(logspace(0,1,100)), //create_grid(0,1e7,1e-2,0.1)),
  sim_(pg_,
       grid_,
       sr_,
       Uniform(1e-9),
       Uniform(1e-6),
       Uniform(0),
       Uniform(0),
       eos_,
       rs_,
       vm_,
       bc_,
       source_)
{
  assert(grid_.size()>1);
}

hdsim1D& SimData::getSim(void)
{
  return sim_;
}
